/***************************************************************************//**
 *   @file   AD7730L.c
 *   @brief  Implementation of AD7730L Driver.
 *   @author DNechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 544
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "AD7730.h"		// AD7730L definitions.
#include "communication.h"	// Communication definitions.

/***************************************************************************//**
 * @brief Configures I/O pins from/towards the component and resets the device.
 *
 * @return none.
*******************************************************************************/
unsigned char AD7730L_Init(void)
{
    AD7730L_RESET_OUT;
    AD7730L_SYNC_OUT;
    AD7730L_RESET_HIGH;
    AD7730L_SYNC_HIGH;
    AD7730L_READY_IN;
    AD7730L_SoftReset();
    
    return(1);
}

/***************************************************************************//**
 * @brief Resets the device.
 *
 * @return none.
*******************************************************************************/
void AD7730L_SoftReset(void)
{
    unsigned char registerWord[6] = {0};
    
    registerWord[0] = 0x01;
    registerWord[1] = 0xFF;
    registerWord[2] = 0xFF;
    registerWord[3] = 0xFF;
    registerWord[4] = 0xFF;
    registerWord[5] = 0xFF;
    SPI_Write(registerWord, 5);
}

/***************************************************************************//**
 * @brief Writes data into a register.
 *
 * @param registerAddress - Address of the register.
 * @param registerValue - Value of the register.
 * @param bytesNumber - Number of bytes to be written.
 *
 * @return none.
*******************************************************************************/
void AD7730L_SetRegisterValue(unsigned char registerAddress,
                              unsigned long registerValue,
                              unsigned char bytesNumber)
{
    unsigned char writeCommand[4] = {0};
    unsigned char* dataPointer    = (unsigned char*)&registerValue;
    unsigned char bytesNr         = bytesNumber;
    
    writeCommand[0] = 0x01;
    writeCommand[1] = AD7730L_COMM_RW(AD7730L_COMM_SINGLE_WRITE) |
                      registerAddress;
    SPI_Write(writeCommand, 1); 
    writeCommand[0] = 0x01;
    while(bytesNr != 0)
    {
        writeCommand[bytesNr] = *dataPointer;
        dataPointer ++;
        bytesNr --;
    }
    SPI_Write(writeCommand, bytesNumber);
}

/***************************************************************************//**
 * @brief Reads the value of a register.
 *
 * @param registerAddress - Address of the register.
 * @param bytesNumber - Number of bytes that will be read.
 *
 * @return buffer - Value of the register.
*******************************************************************************/
unsigned long AD7730L_GetRegisterValue(unsigned char registerAddress,
                                       unsigned char bytesNumber)
{
    unsigned char registerWord[4] = {0}; 
    unsigned long buffer          = 0x0;
    unsigned char i               = 0;
   
    registerWord[0] = 0x01;
    registerWord[1] =  AD7730L_COMM_RW(AD7730L_COMM_SINGLE_READ) | 
                       registerAddress;
    SPI_Write(registerWord, 1);  
    registerWord[0] = 0x01;
    registerWord[1] = 0x00;
    registerWord[2] = 0x00;
    registerWord[3] = 0x00;
    SPI_Read(registerWord, bytesNumber);
    for(i = 0;i < bytesNumber;i ++) 
    {
        buffer = (buffer << 8) + registerWord[i];
    }
    
    return(buffer);
}

/***************************************************************************//**
 * @brief Waits for Ready pin to go low.
 *
 * @return none.
*******************************************************************************/
void AD7730L_WaitForReady(void)
{
	unsigned char data = 0x01;
	
	while( data )
    {
    	AD7730L_READY_STATUS;
    }
}

/***************************************************************************//**
 * @brief Writes the specified mode of operation in Mode Register.
 *
 * @param modeOperation - Mode of Operation.
 * @param inputRange - The range of the input to be measured.
 * @param channel - Selects the channel of the convertor.
 *
 * @return none.
*******************************************************************************/
void AD7730L_SetMode(unsigned short modeOperation, 
                     unsigned char inputRange,
                     unsigned char channel)
{
    unsigned short oldRegisterVal = 0x0;
    unsigned short newRegisterVal = 0x0;
    
    oldRegisterVal = AD7730L_GetRegisterValue(
                                           AD7730L_COMM_ADDR(AD7730L_REG_MODE),
                                           2);
    newRegisterVal = oldRegisterVal & ~(AD7730L_MODE_SEL(0x07) |
                                        AD7730L_MODE_RANGE_SEL(0x03) |
                                        AD7730L_MODE_CHANNEL_SEL(0x03) );
    newRegisterVal = newRegisterVal | 
                     modeOperation |
                     inputRange |
                     channel;
    AD7730L_SetRegisterValue(AD7730L_COMM_ADDR(AD7730L_REG_MODE),
                             newRegisterVal, 
                             2);
}

/***************************************************************************//**
 * @brief Writes a value corresponding to an output rate in Filter Register.
 *
 * @param outputRate - Is a value corresponding to an output rate.
 *                     Example: 2048 - corresponds to 50 Hz Output Rate
 *                                     (chop = 1; skip = 0);
 *                                75 - corresponds to 7.6 kHz Output Rate
 *                                     (chop = 1; skip = 0).
 * @param chop - Value of the AD7730L_FILTER_CHP bit.
 * @param skip - Value of the AD7730L_FILTER_SKIP bit.
 *
 * @return none.
*******************************************************************************/
void AD7730L_SetFilterOutputRate(unsigned long outputRate,
                                 unsigned char chop,
                                 unsigned char skip)
{
    unsigned long oldRegisterVal = 0x0;
    unsigned long newRegisterVal = 0x0;
    
    oldRegisterVal = AD7730L_GetRegisterValue(
                                          AD7730L_COMM_ADDR(AD7730L_REG_FILTER),
                                          3);
    newRegisterVal = oldRegisterVal & ~(AD7730L_FILTER_SINC3_FILTER(0xFFF) |
                                        AD7730L_FILTER_SKIP |
                                        AD7730L_FILTER_CHP);
    newRegisterVal = newRegisterVal |
                     outputRate |
                     (AD7730L_FILTER_SKIP * skip) |
                     (AD7730L_FILTER_CHP * chop);
    AD7730L_SetRegisterValue(AD7730L_COMM_ADDR(AD7730L_REG_FILTER),
                             newRegisterVal, 
                             3);
}

/***************************************************************************//**
 * @brief Performs a calibration sequence for a channel.
 *
 * @param channel - Selects the channel of the convertor.
 * @param inputRange - The range of the input to be measured.
 * @param outputRate - Is a value corresponding to an output rate.
 * @param dacOffset - Code for the offset-compensation DAC on the part.
 *                    Example: 0x20 - 0 mV subtraction value.
 *                             0x23 - 7.5 mV subtraction value, if Vref = 5V.
 *                             0x03 - 7.5 mV addition value, if Vref = 5V.
 *
 * @return none.
*******************************************************************************/
void AD7730L_Calibrate(unsigned char channel,
                       unsigned char inputRange,
                       unsigned long outputRate, 
                       unsigned char dacOffset 
                       )
{
    AD7730L_SetFilterOutputRate(outputRate, 1, 0);
    AD7730L_SetRegisterValue(AD7730L_COMM_ADDR(AD7730L_REG_DAC),
                               dacOffset,
                               1);
    AD7730L_SetRegisterValue(
                         AD7730L_COMM_ADDR(AD7730L_REG_MODE), 
                         AD7730L_MODE_WL | AD7730L_MODE_HIREF,
                         2);
    AD7730L_SetMode(AD7730L_MODE_SEL(AD7730L_MODE_INT_FULL_S_CLB),
                    inputRange, 
                    channel);
    AD7730L_WaitForReady();
    AD7730L_SetMode(AD7730L_MODE_SEL(AD7730L_MODE_INT_ZERO_S_CLB),
                    inputRange, 
                    channel);
    AD7730L_WaitForReady();
}

/***************************************************************************//**
 * @brief Reads the result of a single conversion.
 *
 * @param channel - Selects the channel of the convertor.
 * @param inputRange - The range of the input to be measured.
 *
 * @return convResult - The result of the analog to digital conversion.
*******************************************************************************/
unsigned long AD7730L_SingleRead(unsigned char channel, 
                                 unsigned char inputRange)
{
    unsigned long convResult = 0x0;
    
    AD7730L_SetMode(AD7730L_MODE_SEL(AD7730L_MODE_SINGLE_CONV),
                    inputRange,
                    channel);
    AD7730L_WaitForReady();
    convResult = AD7730L_GetRegisterValue(AD7730L_COMM_ADDR(AD7730L_REG_DATA),
                                          3);
    
    return(convResult);
}

/***************************************************************************//**
 * @brief Returns the average of several conversion results.
 *
 * @param channel - Selects the channel of the convertor.
 * @param inputRange - The range of the input to be measured.
 * @param sampleNumber - The number of consecutive conversions to be made.
 *
 * @return samplesAverage - The average of the conversion results.
*******************************************************************************/
unsigned long AD7730L_ContinuousReadAvg(unsigned char channel,
                                        unsigned char inputRange,
                                        unsigned char sampleNumber)
{
    unsigned char registerWord[4] = {0};
    unsigned long samplesAverage = 0;
    unsigned char count = 0x00;
    
    AD7730L_SetMode(AD7730L_MODE_SEL(AD7730L_MODE_CONT_CONV),
                    inputRange, 
                    channel);
    registerWord[0] = 0x01;
    registerWord[1] = AD7730L_COMM_RW(AD7730L_COMM_START_CONT_RD) |
                      AD7730L_COMM_ADDR(AD7730L_REG_DATA);
    SPI_Write(registerWord, 1);

    for(count = 0;count < sampleNumber;count ++)
    {
        registerWord[0] = 0x01;
        registerWord[1] = 0x0;
        registerWord[2] = 0x0;
        registerWord[3] = 0x0;
        AD7730L_WaitForReady();
        SPI_Read(registerWord, 3);
        samplesAverage += ((unsigned long)registerWord[0] << 16) +
                            ((unsigned long)registerWord[1] << 8) +
                            registerWord[2];
    }
    samplesAverage = samplesAverage / sampleNumber;
    registerWord[0] = 0x01;
    registerWord[1] = AD7730L_COMM_RW(AD7730L_COMM_STOP_CONT_RD) |
                      AD7730L_COMM_ADDR(AD7730L_REG_COMM);
    SPI_Write(registerWord, 1);
    
    return(samplesAverage);
}
